#ifdef LINUX
#include <stdio.h>
#include "SDL.h"
#endif

#include "Hexagon.h"

// Initialise les cellules (centre, voisins...)
Maillage::Maillage(int rayon, int nblignes, int nbcolonnes) {
  int i, j;
  float x, y;
  Rayon = rayon;
  nb_lignes = nblignes;
  nb_colonnes = nbcolonnes;

  RayonRacine3 = Rayon * Racine3;
  cellules = new Cell [ nb_lignes * nb_colonnes ];

  // Initialisation du centre des cellules
  x = Rayon; // 1ere colonne xcentre = Rayon
  for ( j = 0;j < nb_colonnes;j++ ) {
    // Offset en y en fonction de la parite de la colonne
    if ( j%2 == 0 ) y =  RayonRacine3 / 2.0;
    else            y =  RayonRacine3;

    for ( i = 0;i < nb_lignes;i++ ) {
      cellules[ j *nb_lignes + i ].SetCenter( ( int ) x,( int ) y );
      y = y + RayonRacine3; // Pas de Racine3*Rayon
    }
    x = x + ( ( 3.0 * Rayon ) / 2.0 ); // Pas de 3R/2
  }

  // Initialisation des voisins
  int numero;
  int Nord, Sud, NEst, NWest, SEst, SWest;
  numero = 0;
  for ( j = 0;j < nb_colonnes;j++ ) {
    for ( i = 0;i < nb_lignes;i++ ) {
      Nord = NOCELL;
      NEst = NOCELL;
      SEst = NOCELL;
      Sud = NOCELL;
      SWest = NOCELL;
      NWest = NOCELL;

      // voisin Nord
      // Le voisin Nord est la cellule -1 sauf s'il s'agit de
      // la premiere ligne
      if ( i > 0 ) Nord = numero - 1;

      // voisin Sud
      // Le voisin Sud est la cellule +1 sauf s'il s'agit de
      // la derniere ligne
      if ( i < nb_lignes - 1 ) Sud = numero + 1;

      if ( j % 2 == 0 ) { // Colonnes impaires
        // voisin Nord-Est
        // Le voisin Nord-Est est la cellule +nb_lignes-1 sauf s'il
        // s'agit de la 1ere ligne ou de la derniere colonne
        if ( i > 0 && j < nb_colonnes - 1) NEst = numero + nb_lignes -1;

        // voisin Sud-Est
        // Le voisin Sud-Est est la cellule +nb_lignes sauf s'il
        // s'agit de la derniere colonne
        if ( j < nb_colonnes - 1 ) SEst = numero + nb_lignes;

        // voisin Sud-West
        // Le voisin Sud-West est la cellule -nb_lignes sauf s'il
        // s'agit de la 1ere colonne
        if ( j > 0 ) SWest = numero - nb_lignes;

        // voisin Nord-West
        // Le voisin Nord-West est la cellule -nb_lignes-1 sauf s'il
        // s'agit de la 1ere ligne ou de la premiere colonne
        if ( j > 0 && i > 0 ) NWest = numero -nb_lignes -1;
      }
      else { // Colonnes paires
        // voisin Nord-Est
        // Le voisin Nord-Est est la cellule +nb_lignes sauf s'il s'agit
        // de la derniere colonne
        if ( j < nb_colonnes -1 ) NEst = numero + nb_lignes;

        // voisin Sud-Est
        // Le voisin Sud-Est est la cellule +nb_lignes+1 sauf s'il s'agit
        // de la derniere ligne ou de la derniere colonne
        if ( i < nb_lignes -1 && j < nb_colonnes -1 ) SEst = numero + nb_lignes + 1;

        // voisin Sud-West
        // Le voisin Sud-West est la cellule -nb_lignes+1 sauf s'il
        // s'agit de la derniere ligne
        if ( i < nb_lignes -1 ) SWest = numero - nb_lignes + 1;

        // voisin Nord-West
        // Le voisin Nord-West est toujours la cellule -nb_lignes
        NWest = numero - nb_lignes;
      }

      cellules[ numero ].SetNeighbour( Nord, NEst, SEst, Sud, SWest, NWest );
      numero++ ;
    }
  }
}

// Detruit les cellules
Maillage::~Maillage() {
  if ( cellules ) delete [] cellules;
}

int Maillage::GetWidth (void) {
  int ncell_h = (nb_colonnes + 1) / 2;
  int ncell_l = nb_colonnes - ncell_h;

  int largeur = (int) (((2 * ncell_h) + ncell_l) * Rayon);

  if (nb_colonnes %2 == 0) largeur += (int) (Rayon / 2.0); // Ajustement


  largeur += 1;
  return largeur;
}

int Maillage::GetHeight (void) {
  int hauteur = (int) ((Racine3 * Rayon) * (nb_lignes + 0.5));

  hauteur+=1;
  return hauteur;
}

// Pour un point x,y, initialise les indices des droites
// 0, 60 et 120 degres les plus proches du point
void Maillage::NumerosLignes( int xsouris, int ysouris,
                       int &numeroligne0,
                       int &numeroligne60,
                       int &numeroligne120 ) {
  numeroligne0 = ( int ) ( (2.0 * ysouris) / RayonRacine3 );
  numeroligne60 = ( int ) ( (ysouris * 1.0 / RayonRacine3) + (xsouris * 1.0 / Rayon) - 0.5);
  // Pourquoi +120 ??? : Parce que 120 est multiple de 3 ?
  // Ca evite d'avoir des indices negatifs pour les lignes 120 degres
  // Dans ce cas, l'indice de la droite 120 degres passant par la cellule 0 vaut 120 au lieu de 0
  numeroligne120 = ( int )( (xsouris * 1.0 / Rayon) - (ysouris * 1.0 / RayonRacine3) - 0.5 + 120);
}

// Determine les indices des 3 droites se coupant au centre d'une cellule
void Maillage::NumerosLignesCentre( int &p0,
                                 int &n60,
                                 int &m120 ) {
int p, n, m;

  p = p0 % 3;
  n = n60 % 3;
  m = m120 % 3;

  if ( m == 0 ) {
    if ( n == 0 ) {
      if ( p == 0 ) { // m=0, n=0, p=0
        p0++ ; n60++ ;
      }
      else
      if ( p == 1 ) {// m=0, n=0, p=1
        // cout << "Cas m=0, n=0, p=0" << endl;
      }
      else { // m=0, n=0, p=2
        m120++ ;
      }
    }
    else
    if ( n == 1 ) {
      if ( p == 0 ) {// m=0, n=1, p=0
        p0++ ;
      }
      else
      if ( p == 1 ) { // m=0, n=1, p=1
        ;
      }
      else { // m=0, n=1, p=2
        // cout << "Cas m=0, n=1, p=2" << endl;
      }
    }
    else {
      if ( p == 0 ) { // m=0, n=2, p=0
        // cout << "Cas m=0, n=2, p=0" << endl;
      }
      else
      if ( p == 1 ) { // m=0, n=2, p=1
        p0++ ; n60++ ; m120++ ;
      }
      else { // m=0, n=2, p=2
        n60++ ; m120++ ;
      }
    }
  }
  else
  if ( m == 1 ) {
     if ( n == 0 ) {
      if ( p == 0 ) { // m=1, n=0, p=0
        // cout << "Cas m=1, n=0, p=0" << endl;
      }
      else
      if ( p == 1 ) { // m=1, n=0, p=1
        p0++ ;
      }
      else { // m=1, n=0, p=2
        ;
      }
    }
    else
    if ( n == 1 ) {
      if ( p == 0 ) { // m=1, n=1, p=0
        n60++ ; m120++ ;
      }
      else
      if ( p == 1 ) { // m=1, n=1, p=1
        // cout << "Cas m=1, n=1, p=1" << endl;
      }
      else { // m=1, n=1, p=2
        p0++ ; n60++ ; m120++ ;
      }
    }
    else {
      if ( p == 0 ) { // m=1, n=2, p=0
        m120++ ;
      }
      else
      if ( p == 1 ) { // m=1, n=2, p=1
        p0++ ; n60++ ;
      }
      else { // m=1, n=2, p=2
        // cout << "Cas m=1, n=2, p=2" << endl;
      }
    }
  }
  else {
     if ( n == 0 ) {
      if ( p == 0 ) { // m=2, n=0, p=0
        p0++ ; n60++ ; m120++ ;
      }
      else
      if ( p == 1 ) { // m=2, n=0, p=1
        n60++ ; m120++ ;
      }
      else { // m=2, n=0, p=2
        // cout << "Cas m=2, n=0, p=2" << endl;
      }
    }
    else
    if ( n == 1 ) {
      if ( p == 0 ) { // m=2, n=1, p=0
        // cout << "Cas m=2, n=1, p=0" << endl;
      }
      else
      if ( p == 1 ) { // m=2, n=1, p=1
        m120++ ;
      }
      else { // m=2, n=1, p=2
        p0++ ; n60++ ;
      }
    }
    else {
      if ( p == 0 ) { // m=2, n=2, p=0
        ;
      }
      else
      if ( p == 1 ) { // m=2, n=2, p=1
        // cout << "Cas m=2, n=2, p=1" << endl;
      }
      else { // m=2, n=2, p=2
        p0++ ;
      }
    }
  }
}

// Retourne les coordonnees du centre d'une cellule
// a partir des indices de droites 0, 60 et 120 degres
void Maillage::PointXYIntersection( int numeroligne0,
                           int numeroligne60,
                           int &xcentre,
                           int &ycentre ) {
  xcentre = ( int )( (( 1 + 2.0 * numeroligne60 - numeroligne0) * Rayon) / 2.0 );
  ycentre =( int )( (RayonRacine3 * numeroligne0) / 2.0 );
}

// Retourne l'indice de cellule dont le centre est (xcentre,ycentre)
// NOCELL dans le cas ou ce n'est pas une cellule
int Maillage::ReturnCell( int xcentre, int ycentre ) {
  int numeroColonne, numeroLigne;
  int numero;

  numeroColonne = ( int )( (2.0 * xcentre) / (3.0 * Rayon) );
  numeroLigne = ( int )( (ycentre - (RayonRacine3 / 2)) / RayonRacine3);
  numero = nb_lignes * numeroColonne + numeroLigne;

  // On elimine les indices impossibles
  if ( numero < 0 || numero >= nb_lignes * nb_colonnes ) return( NOCELL );
  // On verifie s'il s'agit bien d'une cellule en comparant
  // le centre calcule avec le centre reel de la cellule
  //  dont on a le numero
  if ( cellules[ numero ].GetX() != xcentre || cellules[ numero ].GetY() != ycentre ) {
    return( NOCELL );
  }

  return( numero );
}

// Methode utilisateur pour identifier une cellule
int Maillage::SolveCell( int x, int y ) {
int numeroligne0, numeroligne60, numeroligne120;
int xcentre, ycentre;

  // On elimine le cas ou le curseur est dans la zone inferieure a la droite n60(0)
  if ( ( y * 1.0 / RayonRacine3) + (x * 1.0 / Rayon) - 0.5 < 0 ) return( NOCELL );

  NumerosLignes(x, y, numeroligne0, numeroligne60, numeroligne120);
  NumerosLignesCentre( numeroligne0, numeroligne60, numeroligne120);
  PointXYIntersection(numeroligne0, numeroligne60, xcentre, ycentre );
  return( ReturnCell(xcentre, ycentre) );
}

// Methode utilisateur pour tester l'initialisation des voisins
void Maillage::ShowNeighbour(int numero) {
#ifdef LINUX
  printf ("Activation cellule : %d\n", numero);
  if ( numero != NOCELL )
  {
    printf ("Nord %d\n", cellules[ numero ].GetNo());
    printf ("Nord Est %d\n", cellules[ numero ].GetNE());
    printf ("Sud Est %d\n", cellules[ numero ].GetSE());
    printf ("Sud %d\n", cellules[ numero ].GetSu());
    printf ("Sud West %d\n", cellules[ numero ].GetSW());
    printf ("Nord West %d\n", cellules[ numero ].GetNW());
  }
#endif
}

// Dessine la grille complete
void Maillage::DrawMaillage (gfxbitmap *bitmap, UInt32 color) {
  int indice;
  for ( indice = nb_lignes *nb_colonnes -1; indice >= 0;indice-- )
    cellules[ indice ].DrawCell(bitmap, Rayon, color);
}

void Maillage::DrawCell (int cellule_n, gfxbitmap *bitmap, UInt32 color) {
  if (cellule_n != -1 && cellule_n < (nb_lignes*nb_colonnes)) {
    cellules[ cellule_n ].DrawCell(bitmap, Rayon, color);
  }
}

// Dessine une cellule
void Cell::DrawCell (gfxbitmap *bitmap, int rayon, UInt32 color) {

  float RayonRacine3Sur2;

    RayonRacine3Sur2 = ( rayon * Racine3 ) / 2.0;

    UInt16 x1 = xcentre + rayon;
    UInt16 y1 = ycentre;
    UInt16 x2 = xcentre + ( UInt16 ) ( rayon / 2.0 );
    UInt16 y2 = ycentre + ( UInt16 ) ( RayonRacine3Sur2 );
    UInt16 x3 = xcentre - ( UInt16 ) ( rayon / 2.0 );
    UInt16 y3 = ycentre + ( UInt16 ) ( RayonRacine3Sur2 );
    UInt16 x4 = xcentre - rayon;
    UInt16 y4 = ycentre;
    UInt16 x5 = xcentre - ( UInt16 ) ( rayon / 2.0 );
    UInt16 y5 = ycentre - ( UInt16 ) ( RayonRacine3Sur2 );
    UInt16 x6 = xcentre + ( UInt16 ) ( rayon / 2.0 );
    UInt16 y6 = ycentre - ( UInt16 ) ( RayonRacine3Sur2 );

    GfxLine(bitmap, x1, y1, x2, y2, color);
    GfxLine(bitmap, x2, y2, x3, y3, color);
    GfxLine(bitmap, x3, y3, x4, y4, color);
    GfxLine(bitmap, x4, y4, x5, y5, color);
    GfxLine(bitmap, x5, y5, x6, y6, color);
    GfxLine(bitmap, x6, y6, x1, y1, color);
}

// Initialise les coordonnees du centre d'une cellule
void Cell::SetCenter( int x, int y ) {
  xcentre = x;
  ycentre = y;
}

// Constructeur de celllule
Cell::Cell() {
  No = NOCELL;
  NE = NOCELL;
  SE = NOCELL;
  Su = NOCELL;
  SW = NOCELL;
  NW = NOCELL;
}

// Methode appelee par le constructeur de board
// lors de l'initialisation des voisins
void Cell::SetNeighbour( int nord, int nordest, int sudest, int sud, int sudwest, int nordwest ) {
  No = nord;
  NE = nordest;
  SE = sudest;
  Su = sud;
  SW = sudwest;
  NW = nordwest;
}

int main () {
  GfxInit();
  GfxClear(0, GfxColor(0,0,0));

  Maillage *theMaillage = new Maillage (16, 10, 9);
  UInt32 color = GfxColor(0x20, 0x50, 0x20);
  theMaillage->DrawMaillage (0, color); // 0 : Screen
  GfxLocate(39, 5);
  GfxString ("Cliquer pour quitter", GfxColor(0xC0, 0xC0, 0xC0),GfxColor(0,0,0) );
  GfxUpdate ();
  int cellule = -1;
  int old_cellule = -1;
  while (1) {
    UInt16 mx, my;
    GfxGetMouseXY (&mx, &my);
    cellule = theMaillage->SolveCell (mx, my);
    if (old_cellule != cellule) {
      if (old_cellule != -1) {
        theMaillage->DrawCell (old_cellule, 0, color);
      }

      if (cellule != -1) {
        theMaillage->DrawCell (cellule, 0, GfxColor(0x00, 0xD0, 0x20));
      }
      old_cellule = cellule;
      GfxUpdate();
    }

    if (GfxGetMouseButton ()) break;
  }

  delete theMaillage;

  GfxQuit();
  return 0;
}
