/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>

#include "AWB.h"
#include "AWD.h"

AWB::AWB (AWD *aParent, int ax, int ay, int aw, int ah, UInt8 Id, const char *atext) : AWVB (aParent, ax,ay,aw,ah, Id) {
  strcpy (text, atext);
}

AWB::~AWB () {
}

bool AWB::ProcessEvent (int mouse_move, UInt16 mx, UInt16 my, 
						int mouse_button, bool mb1, bool mb2,
						EVT &evt) {
  if (mouse_button) {
    if (mb1 == true) {
	  evt = Pressed ();
	}
    else {
	  evt = UnPressed ();
	}
	return true;
  }

  return false;
}

void AWB::Draw () {
  GfxFillRectangle(0,x,y,w,h,background_color); // All the area
  UInt16 ws, hs;
  GfxStringWidth (text, ws, hs); 

  if (IsPressed ()) {
    // Borders
    GfxLine(0,x,y,x+w-1,y, shadow_color_dark);
    GfxLine(0,x,y+h-1,x+w-1,y+h-1, shadow_color_light);
    GfxLine(0,x,y,x,y+h-1, shadow_color_dark);
    GfxLine(0,x+w-1,y,x+w-1,y+h-1, shadow_color_light);

    GfxSetClip(0,x,y,x+w-1,y+h-1);
    GfxStringXY(0, text, x+((w-ws)/2)+2, y+((h-hs)/2)+2, foreground_color, background_color);
    GfxSetClip(0,0,0,0,0);
  }
  else { // Not pressed
    // Borders
    GfxLine(0,x,y,x+w-1,y, shadow_color_light);
    GfxLine(0,x,y+h-1,x+w-1,y+h-1, shadow_color_dark);
    GfxLine(0,x,y,x,y+h-1, shadow_color_light);
    GfxLine(0,x+w-1,y,x+w-1,y+h-1, shadow_color_dark);

    GfxSetClip(0,x,y,x+w-1,y+h-1);
    GfxStringXY(0, text, x+((w-ws)/2), y+(h/2)-(hs/2), foreground_color, background_color);
    GfxSetClip(0,0,0,0,0);
  }
}

void AWB::UnDraw () {
}

EVT AWB::Leave () {
  if (IsPressed()) {
	UnPressed ();
	EVT evt (GetId(), EVT_UNPRESSED);
	return evt;
  }
  else {
	EVT evt;
	return evt;
  }
}

// End of file
