/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>

#include "AWI.h"
#include "AWD.h"

AWI::AWI (AWD *aParent, int ax, int ay, int aw, int ah, UInt8 Id, 
	  const char *unpressedtilename, 
		  const char *pressedtilename) : AWVB (aParent, ax,ay,aw,ah,Id) {
  pressedtile = GfxCreateBitmapFromGif(pressedtilename);
  unpressedtile = GfxCreateBitmapFromGif(unpressedtilename);
}

AWI::~AWI () {
  if (unpressedtile) GfxDeleteBitmap(unpressedtile);
  if (pressedtile) GfxDeleteBitmap(pressedtile);
}

bool AWI::ProcessEvent (int mouse_move, UInt16 mx, UInt16 my, 
						int mouse_button, bool mb1, bool mb2,
						EVT& evt) {
  if (mouse_button) {
    if (mb1==true) {
	  evt = Pressed ();
	}
    else {
	  evt = UnPressed ();
	}
	return true;
  }

  return false;
}

void AWI::Draw () {
  GfxFillRectangle(0,x,y,w,h,background_color); // All the area

  // Borders
  if (IsPressed()) {
    GfxLine(0,x,y,x+w-1,y, shadow_color_dark);
    GfxLine(0,x,y+h-1,x+w-1,y+h-1, shadow_color_light);
    GfxLine(0,x,y,x,y+h-1, shadow_color_dark);
    GfxLine(0,x+w-1,y,x+w-1,y+h-1, shadow_color_light);
    if (pressedtile) {
      GfxSetClip(0,x+1,y+1,x+w-2,y+h-2);
      GfxBlit(pressedtile,0,0,0,x+1,y+1,pressedtile->w,pressedtile->h);
      GfxSetClip(0,0,0,0,0);
    }
  }
  else { // Not pressed
    GfxLine(0,x,y,x+w-1,y, shadow_color_light);
    GfxLine(0,x,y+h-1,x+w-1,y+h-1, shadow_color_dark);
    GfxLine(0,x,y,x,y+h-1, shadow_color_light);
    GfxLine(0,x+w-1,y,x+w-1,y+h-1, shadow_color_dark);
    if (unpressedtile) {
      GfxSetClip(0,x+1,y+1,x+w-2,y+h-2);
      GfxBlit(unpressedtile,0,0,0,x+1,y+1,unpressedtile->w,unpressedtile->h);
      GfxSetClip(0,0,0,0,0);
    }
  }
}

void AWI::UnDraw () {
}

EVT AWI::Leave () {
  if (IsPressed()) {
	UnPressed ();
	EVT evt (GetId(), EVT_UNPRESSED);
	return evt;
  }
  else {
	EVT evt (GetId(), EVT_NONE);
	return evt;
  }
}
// End of file
