/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// TODO
// Rendre insensible les boutons (devalides)

#include <stdio.h>
#include <iostream>

#include "AW.h"
#include "AWA.h" // Application
#include "AWRD.h" // Root Dialog
#include "AWVB.h"// Base for the buttons
#include "AWB.h" // Button
#include "AWG.h" // Gauge
#include "AWL.h" // Label
#include "AWC.h" // Check button
#include "AWI.h" // Icon button

#include "spectrum_logo.h"
#define ID_BUTTON_NONE          0 

#define ID_BUTTON_MAIN_DIAG     101
#define ID_BUTTON_MAIN_POWER    102
#define ID_BUTTON_MAIN_TEMP     103
#define ID_BUTTON_MAIN_PARAM    104
#define ID_BUTTON_MAIN_HELP     105

#define ID_BUTTON_DIAG_CLOSE     110
#define ID_BUTTON_POWER_CLOSE    120
#define ID_BUTTON_TEMP_CLOSE     130
#define ID_BUTTON_PARAM_CLOSE    140
#define ID_BUTTON_HELP_CLOSE     150

class MainApplication : public AWA {
public:
  MainApplication();
  virtual ~MainApplication();
  
private:
  void Process ();
  
  // Dialogues et leurs objects
  AWD *_dialogMain;
  AWB *_buttonMainDiag;
  AWB *_buttonMainPower;
  AWB *_buttonMainTemp;
  AWB *_buttonMainParam;
  AWB *_buttonMainHelp;
  
  AWD *_dialogDiag;
  AWL *_labelDiag;
  AWB *_buttonDiagClose;
  
  AWD *_dialogPower;
  AWL *_labelPower;
  AWB *_buttonPowerClose;

  AWD *_dialogTemp;
  AWL *_labelTemp;
  AWB *_buttonTempClose;

  AWD *_dialogParam;
  AWL *_labelParam;
  AWB *_buttonParamClose;

  AWD *_dialogHelp;
  AWL *_labelHelp;
  AWB *_buttonHelpClose;
};

MainApplication::MainApplication () {
  // Main Dialog
  _dialogMain = new AWRD ();
  _buttonMainDiag = new AWB (_dialogMain, 5,50,200,20, ID_BUTTON_MAIN_DIAG, "Diagnostics ...");
  _buttonMainPower = new AWB (_dialogMain, 5,80,200,20, ID_BUTTON_MAIN_POWER, "Power supplies ...");
  _buttonMainTemp = new AWB (_dialogMain, 5,110,200,20, ID_BUTTON_MAIN_TEMP, "Temperatures ...");
  _buttonMainParam = new AWB (_dialogMain, 5,140,200,20, ID_BUTTON_MAIN_PARAM, "Parameters ...");
  _buttonMainHelp = new AWB (_dialogMain, 5,170,200,20, ID_BUTTON_MAIN_HELP, "Help ...");

  // Diagnostics
  _dialogDiag =  new AWRD ();
  _labelDiag = new AWL (_dialogDiag, 40, 10, 100, 20, "DIAGNOSTICS"); 
  _buttonDiagClose = new AWB (_dialogDiag, 5,280,200,20, ID_BUTTON_DIAG_CLOSE, "Close diagnostics");

  // Power
  _dialogPower =  new AWRD ();
  _labelPower = new AWL (_dialogPower, 40, 10, 100, 20, "POWER SUPPLIES"); 
  _buttonPowerClose = new AWB (_dialogPower, 5,280,200,20, ID_BUTTON_POWER_CLOSE, "Close power supplies");
  

  // Temperatures
  _dialogTemp =  new AWRD ();
  _labelTemp = new AWL (_dialogTemp, 40, 10, 100, 20, "TEMPERATURES"); 
  _buttonTempClose = new AWB (_dialogTemp, 5,280,200,20, ID_BUTTON_TEMP_CLOSE, "Close temperatures");

  // Parameters
  _dialogParam =  new AWRD ();
  _labelParam = new AWL (_dialogParam, 40, 10, 100, 20, "PARAMETERS"); 
  _buttonTempClose = new AWB (_dialogParam, 5,280,200,20, ID_BUTTON_PARAM_CLOSE, "Close parameters");

  // Help
  _dialogHelp =  new AWRD ();
  _labelHelp = new AWL (_dialogHelp, 40, 10, 100, 20, "HELP"); 
  _buttonHelpClose = new AWB (_dialogHelp, 5,280,200,20, ID_BUTTON_HELP_CLOSE, "Close help");
  
  // For managment
  AttachDialog (_dialogMain);
  AttachDialog (_dialogDiag);
  AttachDialog (_dialogPower);
  AttachDialog (_dialogTemp);
  AttachDialog (_dialogParam);
  AttachDialog (_dialogHelp);

  // Showing de main
  _dialogMain->Show ();
  
}

MainApplication::~MainApplication () {
}

void MainApplication::Process () {
  EVT evt;
  if (GetEvent(evt)) {
	int id = evt.GetId();
    if (evt.GetType() != EVT_PRESSED) return;

	// Affiche le dialog Help
	if (id == ID_BUTTON_MAIN_DIAG) {
      _dialogMain->Hide();
      _dialogDiag->Show();
      return;
	}

	if (id == ID_BUTTON_MAIN_POWER) {
      _dialogMain->Hide();
      _dialogPower->Show();
      return;
	}

	if (id == ID_BUTTON_MAIN_TEMP) {
      _dialogMain->Hide();
      _dialogTemp->Show();
      return;
	}

	if (id == ID_BUTTON_MAIN_PARAM) {
      _dialogMain->Hide();
      _dialogParam->Show();
      return;
	}

	if (id == ID_BUTTON_MAIN_HELP) {
      _dialogMain->Hide();
      _dialogHelp->Show();
      return;
	}

	// Ferme le dialog
    if (id == ID_BUTTON_DIAG_CLOSE) {
      _dialogDiag->Hide();
      _dialogMain->Show();
      return;
	}

	// Ferme le dialog
    if (id == ID_BUTTON_POWER_CLOSE) {
      _dialogPower->Hide();
      _dialogMain->Show();
      return;
	}

    // Ferme le dialog
    if (id == ID_BUTTON_TEMP_CLOSE) {
      _dialogTemp->Hide();
      _dialogMain->Show();
      return;
	}

    // Ferme le dialog
    if (id == ID_BUTTON_PARAM_CLOSE) {
      _dialogParam->Hide();
      _dialogMain->Show();
      return;
	}

    // Ferme le dialog
    if (id == ID_BUTTON_HELP_CLOSE) {
      _dialogHelp->Hide();
      _dialogMain->Show();
      return;
	}
  }
}

int main () {
  GfxInit(); // Graphic initialization
  GfxClear(0, GfxColor(0, 0, 0));
  MainApplication app; // All the application
  app.MainLoop ();
  return 0;
}
